@@ -16,6 +16,7 @@ from __future__ import print_function
 from __future__ import absolute_import
 import os
 import sys
+import ssl
 import traceback
 from distutils.spawn import find_executable
 import xenv
@@ -24,18 +25,20 @@ import LbPlatformUtils.describe
 from LbPlatformUtils import host_supports_tag, get_viable_containers
 
 from LbEnv.ProjectEnv.lookup import (
-    getEnvXmlPath,
-    findProject,
-    findDataPackage,
-    getLCGRelocation,
-    getHepToolsInfo,
-    NotFoundError,
-    getProjectNameVersion,
-    findLCGForExt,
-    listVersions,
-    listPlatforms,
     EXTERNAL_PROJECTS,
+    findDataPackage,
+    findLCGForExt,
+    findProject,
+    getEnvXmlPath,
+    getHepToolsInfo,
+    getLCGRelocation,
+    getProjectNameVersion,
+    getProjectsDirs,
+    getPackagesDirs,
     InvalidNightlySlotError,
+    listPlatforms,
+    listVersions,
+    NotFoundError,
 )
 from LbEnv.ProjectEnv.version import (
     isValidVersion,
@@ -155,7 +158,7 @@ def localNightlyHelp(prog, err, project, platform, args, error=True):
     import re
     from six.moves.urllib.parse import quote
     from six.moves.urllib.request import urlopen
-    from six.moves.urllib.error import HTTPError
+    from six.moves.urllib.error import HTTPError, URLError
     from ssl import SSLError
     from json import load
     from difflib import get_close_matches
@@ -186,13 +189,18 @@ def localNightlyHelp(prog, err, project, platform, args, error=True):
     # build_id is meant to contain the numeric build id for the requested build
     build_id = err.build_id
     # check if the user request make any sense
+    ctx = ssl.create_default_context(
+        capath="/cvmfs/lhcb.cern.ch/etc/grid-security/certificates/"
+    )
     try:
         # does the slot name exist?
         all_slots = set(
             entry["key"]
             for entry in load(
                 urlopen(
-                    COUCHDB_ROOT + "_design/names/_view/slots?group=true", timeout=5
+                    COUCHDB_ROOT + "_design/names/_view/slots?group=true",
+                    timeout=5,
+                    context=ctx,
                 )
             ).get("rows", [])
         )
@@ -228,6 +236,7 @@ def localNightlyHelp(prog, err, project, platform, args, error=True):
                     COUCHDB_ROOT
                     + '_design/summaries/_view/byDay?key="{}"'.format(build_date),
                     timeout=5,
+                    context=ctx,
                 )
             ).get("rows", [])
             # was the slot built on that date?
@@ -244,7 +253,11 @@ def localNightlyHelp(prog, err, project, platform, args, error=True):
         # at this point we have a numeric id, we try to get the build info
         try:
             slot_info = load(
-                urlopen("{}{}.{}".format(COUCHDB_ROOT, err.slot, build_id), timeout=5)
+                urlopen(
+                    "{}{}.{}".format(COUCHDB_ROOT, err.slot, build_id),
+                    timeout=5,
+                    context=ctx,
+                )
             )
         except HTTPError:
             # the numeric id is not known in the DB
@@ -260,7 +273,7 @@ def localNightlyHelp(prog, err, project, platform, args, error=True):
 
     except SlotError as reason:
         msg = str(reason)
-    except SSLError:
+    except (SSLError, URLError):
         # ignore connection problems and assume it's OK
         pass
 
@@ -427,9 +440,17 @@ class LbRun(xenv.Script):
         parser.add_option(
             "--allow-containers",
             action="store_true",
+            default=True,
+            dest="allow_containers",
             help="Allow a container technology to be automatically chosen if "
             "native support is not available",
         )
+        parser.add_option(
+            "--disallow-containers",
+            action="store_false",
+            dest="allow_containers",
+            help="Prevent falling back to running with a container",
+        )
 
         parser.add_option(
             "--prefer-container",
@@ -916,6 +937,34 @@ class LbRun(xenv.Script):
 
         self._handle_externals(lcg_relocation.get("LCG_external"), manifest)
 
+        # set relocation roots for projects...
+        self.opts.actions.extend(
+            (
+                "set",
+                (
+                    "{}_PROJECT_ROOT".format(p.upper()),
+                    str(os.path.dirname(os.path.dirname(d))),
+                ),
+            )
+            for p, d in getProjectsDirs(
+                self.project,
+                self.version,
+                self.opts.platform,
+                allow_empty_version=self.allow_empty_version,
+            ).items()
+        )
+        # ... and data packages
+        data_packages = getPackagesDirs(
+            self.project,
+            self.version,
+            self.opts.platform,
+            allow_empty_version=self.allow_empty_version,
+        )
+        for p, d in data_packages.items():
+            self.opts.actions.append(
+                ("set", ("{}ROOT".format(p.rsplit("/", 1)[-1].upper()), str(d)))
+            )
+
         # now we can expand project name and version if --path-to-project
         if self.opts.path_to_project:
             self.project, self.version = getProjectNameVersion(
@@ -948,6 +997,11 @@ class LbRun(xenv.Script):
                 p = self.project
             self.opts.actions.append(("loadXML", (p + ".xenv",)))
 
+        # make sure we load the data packages too
+        for p, d in data_packages.items():
+            self.opts.actions.append(
+                ("loadXML", (str("{}/{}.xenv".format(d, p.replace("/", "_"))),))
+            )
         # handle the extra data packages
         for pkg_name, pkg_vers in map(decodePkg, self.opts.use):
             xml_name = pkg_name.replace("/", "_") + ".xenv"
@@ -1022,7 +1076,15 @@ class LbRun(xenv.Script):
             # PATH might be changed when launching the environment so use an absolute filename
             singularity_exe = find_executable("singularity")
             self.log.debug("Using singularity at: %s", singularity_exe)
-            wrapper = [singularity_exe, "exec", "--bind", "/cvmfs", "--userns"]
+            wrapper = [
+                singularity_exe,
+                "exec",
+                "--bind",
+                "/cvmfs",
+                "--bind",
+                os.getcwd(),
+                "--userns",
+            ]
             if "X509_USER_PROXY" in os.environ:
                 wrapper += ["--bind", os.environ["X509_USER_PROXY"]]
 
@@ -1088,7 +1150,7 @@ def main():
     if "--profile" in sys.argv:
         from LbEnv.ProjectEnv.profiling import run
 
-        run("LbRun().main()")
+        run("from LbEnv.ProjectEnv.script import LbRun; LbRun().main()")
     else:
         # special handling of the '#!' case (LBCORE-782)
         # - we must have at least one argument and the first one must be a file
